#! /usr/bin/env python
'''
Module Name:    Utils.py

Description:    Contains helper functions 

Copyright:      Toshiba Confidential

                OCO Source Materials

                Copyright (C) 2012,2013 Toshiba Global Commerce Solutions, Inc.

                The source code for this program is not published
                or otherwise divested of its trade secrets,
                irrespective of what has been deposited with the
                U. S. Copyright Office.

Initial Author: Francisco Liwa

Function:

Notes:

Dependencies:    None

Restrictions:    None

Last Modified: 30 September 2013

'''
import os
import sys
import subprocess
import time

#importing helper modules 
from monitorupgrade import TLog as TLOG

#Globals
FILE_FOUND = 1
FILE_NOT_FOUND = 0
EXIT_SUCCESS = 0
EXIT_FAILURE = 1 
UNKNOWN = -1
NO_MONITORS_ATTACHED = 0


#Error Codes
ERROR_OCULAR_UPDATE_FAILED = 2
ERROR_UPDATE_FAILED = 3
ERROR_CODE_DEVICE_NOT_EXIST = 4
ERROR_CODE_OCULAR_DEVICE_NOT_EXIST = 5
ERROR_CODE_DWU_DEVICE_NO_UPDATE = 6
ERROR_CODE_DWU_DEVICE_NOT_CONNECTED = 9
ERROR_CODE_DWU_DEVICE_NOT_FOUND = 10
ERROR_CODE_OPERATION_FAILED = 11
ERROR_CODE_AIP_SEGFAULT = 12
ERROR_DWU_VIDEO_UPDATE_FAILED = 20
ERROR_CODE_INVALID_PARAM = 21
ERROR_RETRY = 99
ERROR_SEGFAULT = 139

#Error Strings
ERROR_STR_DEVICE_NOT_EXIST = 'Device not exist'
ERROR_STR_OPERATION_FAILED = 'Operation failed'
ERROR_STR_INVALID_PARAM = 'Invalid Parameter'
ERROR_STR_Runtime = 'Runtime error'

MUS_LOG_FILE_DIR = '/var/log/toshiba/mus'

#sampo stuffs here
SAMPO_UTIL = '/opt/toshiba/mus/op_utils/sampo/4820MonUtil'
RECOVERY_FILE_SUSE = '/etc/init.d/after.local'
RECOVERY_STR = '4820MonUtil'

#ELO Stuffs here    
ELO_DOWNLOAD = '/opt/toshiba/mus/op_utils/elo/EloDownload.sh'
ELO_CONFIG = '/opt/toshiba/mus/op_utils/elo/EloConfig.sh'
ELO_RECOVERY_FILE = '/opt/toshiba/mus/op_utils/elo/recovery.txt'
OCULAR_RECOVERY_FILE = '/opt/toshiba/mus/op_utils/mxtapp/recovery.txt'
AIP_RECOVERY_FILE = '/opt/toshiba/mus/op_utils/aipfwupdate/recovery.txt'
BOOT_COUNTER ='/tmp/reboot_count'

#Ocular
MXT_APP = '/usr/bin/mxt-app'
BOOT_LOADER_STR='03EB:211D'

#==========================================================================================================
'''
doesFileExist - Returns True if the file exist, False otherwise
'''
#==========================================================================================================

def doesFileExist(filename):
    
    if os.path.exists(filename) and os.path.isfile(filename) and os.access(filename, os.R_OK):
        #print filename + ' exists and is readable'
        TLOG.logger.info('doesFileExist()-Found! :'+ filename)
        return True
    else:
        #print filename  + ' does not exist'
        TLOG.logger.info('doesFileExist()-Not Found! :'+ filename)
        #raise IOError
        return False
#==========================================================================================================
'''
runShell - run any shell command
'''
#==========================================================================================================
def runShell(cmd,cwd_= None):
    
    #TLOG.logger.info(cmd)    
    process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,cwd=cwd_)
    while True:
        line = process.stdout.readline()
        #print line
        if not line:
            break
        TLOG.logger.info(line)    
    
    process.wait()
    return process.returncode



#==========================================================================================================
'''
runSubprocess - This will call the subprocess module to run the command in the cmd string
'''
#==========================================================================================================
def runSubprocess(cmd,proc,cwd_=None,time_out=60):
    
    TLOG.logger.info( str(cmd) )
    #std_err = None
    #try:
    process = subprocess.Popen(cmd,shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE, cwd = cwd_ )        
    
    while True:
        line = process.stdout.readline()
        #print line
        if not line:
            break
        line = line.strip('\n')
        TLOG.logger.info('        Subprocess[' + proc + '] OUTPUT-->' + ''  +  line )
        msg = ''    
        if (proc == 'SampoVideoFlashTask_4820_21x51x' or proc == 'SampoVideoFlashTask_4820_2Lx5Lx'):
            if "DATA" in line:
                idx = line.find(">")
                for c in line[idx + 1:len(line)]:
                    if c == '<':
                        break
                    msg += c
                print msg + '\r'  
    
    process.wait()
    ret = process.returncode
    
    try:
        if ret == 139 and proc == 'ELO_RECOVER':               # TODO: ELO Recovery Success but with Segmentation Fault
            return EXIT_SUCCESS
        
        elif (ret == 20 ) :                                    # TODO: Error code after Successfully loading Atmel Configuration - Need to verify with Atmel
            return EXIT_SUCCESS
        
        elif (ret == ERROR_CODE_DWU_DEVICE_NO_UPDATE ) :
                #print 'DWU: No Update is Needed'
                return EXIT_SUCCESS
        
        elif ret != EXIT_SUCCESS:
                  
            #TLOG.logger.error('return code:' + str(ret) )
            
            #when the monitor is unplugged during Firmware update.
            if ( ret == ERROR_CODE_DEVICE_NOT_EXIST  or ret == ERROR_UPDATE_FAILED  or ret == ERROR_CODE_DWU_DEVICE_NOT_CONNECTED
                 or ret == ERROR_CODE_DWU_DEVICE_NOT_FOUND or ret == ERROR_OCULAR_UPDATE_FAILED or ret == ERROR_DWU_VIDEO_UPDATE_FAILED ) :  
                
                print '[TaskMgr::runSubprocess()] Error:' + ERROR_STR_Runtime
                raise RuntimeError('Device not exist')
            elif (ret == ERROR_CODE_DWU_DEVICE_NOT_FOUND or ret == ERROR_CODE_OCULAR_DEVICE_NOT_EXIST) :  
                return ERROR_RETRY
               
            elif (ret == ERROR_CODE_OPERATION_FAILED ) :
                print '[TaskMgr::runSubprocess()] Error:' + ERROR_STR_OPERATION_FAILED
            
            elif (ret == ERROR_CODE_INVALID_PARAM ) :
                print '[TaskMgr::runSubprocess()] Error:' + ERROR_STR_INVALID_PARAM
            
            #FLIWA - Added for segfault error with libusb when there are 4820 and 6149 monitors attached
            elif (ret == ERROR_SEGFAULT or ret == ERROR_CODE_AIP_SEGFAULT ) :
                TLOG.logger.info('ERROR: SEGFAULT')
                return ERROR_CODE_AIP_SEGFAULT
                
            elif (proc != 'ELOCONFIGUTILITY'):
                print '[TaskMgr::runSubprocess()] Error:' + str(ret)
                #raise IOError(ret)
                raise RuntimeError(ret)
            #else:
                #raise IOError(ret)
        else:
            
            return EXIT_SUCCESS
    
    except (ValueError):
        
        extype, value = sys.exc_info()[:2]
        print 'Value Error'     
        print >> sys.stderr, "Invalid argument in Popen"
        TLOG.logger.error("%s (%s)" % (extype, value) )
        # return 2 for "bad command line option"
        sys.exit(2)
    
    except (IOError):
        
        extype, value = sys.exc_info()[:2]
        print 'IO Error'     
        #print >> sys.stderr, "Error in process: %s" % proc 
        TLOG.logger.error("TaskMgr::runSubprocess()-[Error in process: %s] %s (s%)" % ( proc,repr(extype), repr(value) ) )
 
        cleanUp(False)
        
        return EXIT_FAILURE
        
    except (RuntimeError):      
       
        value = sys.exc_info()[:2]
        #print value
        print 'Runtime Error'
        #print >> sys.stderr, "TaskMgr::runSubprocess() - Error in process: %s" % proc 
        TLOG.logger.error("TaskMgr::runSubprocess()-[RuntimeError in process: [%s] [%s]" % ( proc, repr(value) ) )
        
        #if ( proc == 'SampoVideoFlashTask_4820_2Lx5Lx' or proc == 'SampoVideoFlashTask_4820_21x51x' ):
            #cleanUp(True)       # Sets the recovery value to True
        #elif (proc == 'ELODOWNLOAD-4820_2Lx5Lx' or proc == 'ELODOWNLOAD-4820_21x51x') :
            #cleanUp(False)
            
            
        return EXIT_FAILURE   
        
            
    except (OSError):
        
        extype, value = sys.exc_info()[:2]
        print 'OS Error'     
        #print >> sys.stderr, "ERROR: %s (%s)" % (extype, value)
        TLOG.logger.error("%s (%s)" % (extype, value) )
        # return 2 for "bad command line option"
        sys.exit(2)
    
    except:
        # all other exceptions get caught here
        extype, value = sys.exc_info()[:2]
        print 'CatchAll Error'
        #print >> sys.stderr, "ERROR: %s (%s)" % (extype, repr(value) )
        TLOG.logger.error("%s (%s)" % (extype,  repr(value) ) )
        sys.exit(1)

# find all 614x monitors in bootloader mode
def find614Xin211Dmode():
        
        mon_list=[]
        query_cmd = MXT_APP  + ' -q > /tmp/count ' 
        process = subprocess.Popen(query_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        process.wait()                  
    
        #mon_list = []
        f = open('/tmp/count', 'r+')
        lines = f.readlines()
        f.close() 
        
        process = subprocess.Popen('rm /tmp/count ', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        process.wait() 
        
        for line in lines:
            line = line.strip('\n')
            if BOOT_LOADER_STR in line:
                usb_id, chip, vid_pid = line.split(' ')
                print 'Device in Boot loader mode:'+ usb_id, chip, vid_pid
                mon_list.append(usb_id)
        
        return mon_list
    
def find614X():
        
        mon_list=[]
        query_cmd = MXT_APP  + ' -q > /tmp/count ' 
        process = subprocess.Popen(query_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        process.wait()                  
    
        #mon_list = []
        f = open('/tmp/count', 'r+')
        lines = f.readlines()
        f.close() 
        
        process = subprocess.Popen('rm /tmp/count ', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        process.wait() 
        
        for line in lines:
            line = line.strip('\n')
            usb_id, chip, vid_pid = line.split(' ')
            print 'Device recovered:'+ usb_id, chip, vid_pid
            mon_list.append(usb_id)
        
        return mon_list           
     
#==========================================================================================================
# cleanUp() -  delete firmware files from the fw_temp directory which MUS checks
#           -  delete updatask.conf configuration file
#==========================================================================================================        
def cleanUp(in_recovery):
       
        #cmd = []
        
        TLOG.logger.info('cleanUp()------------------START---------------')    
        
        if (not in_recovery):
                 
            if ( doesFileExist(ELO_RECOVERY_FILE) ) :
                cmd1 = 'rm' + ' ' + ELO_RECOVERY_FILE + ' ' + OCULAR_RECOVERY_FILE + ' ' + AIP_RECOVERY_FILE
                #cmd.append(cmd1)
                TLOG.logger.info(cmd1)
            
                print cmd1
                rem_proc = runShell(cmd1)
            
            if ( doesFileExist(OCULAR_RECOVERY_FILE) ) :
                cmd1 = 'rm' + ' ' + OCULAR_RECOVERY_FILE 
                #cmd.append(cmd1)
                TLOG.logger.info(cmd1)
            
                print cmd1
                rem_proc = runShell(cmd1)
         
            if ( doesFileExist(AIP_RECOVERY_FILE) ) :
                cmd1 = 'rm' +  ' ' + AIP_RECOVERY_FILE
                #cmd.append(cmd1)
                TLOG.logger.info(cmd1)
            
                print cmd1
                rem_proc = runShell(cmd1)
             
           
        else:
            
            if ( doesFileExist(ELO_RECOVERY_FILE) or doesFileExist(OCULAR_RECOVERY_FILE) or doesFileExist(AIP_RECOVERY_FILE) ) :
                cmd1 = 'rm' + ' ' + ELO_RECOVERY_FILE + ' ' + OCULAR_RECOVERY_FILE + ' ' + AIP_RECOVERY_FILE
                #cmd.append(cmd1)
                TLOG.logger.info(cmd1)
            
                print cmd1
                rem_proc = runShell(cmd1)
                if rem_proc == 0:
                    runShell('reboot')
                    
        TLOG.logger.info('cleanUp()-------------------END----------------\n')     
        
        
def doEloRecover(fw_path):
        
        TLOG.logger.info('doEloRecover() -----------------START---------------')
        eloRecoverCmd = 'sh ' +  ELO_DOWNLOAD + '  -select=fix  -firmware_file=' + str(fw_path)
        print eloRecoverCmd
        if doesFileExist(ELO_DOWNLOAD) != True:
            print 'EloAnakFlashTask::run()-No ELO Download script found!'
            TLOG.logger.error('Elo4820_21x51xFlashTask::run()-No ELO Download script found!')
            raise IOError('No ELO Download script found!')
        else:
            ret = runSubprocess(eloRecoverCmd,'ELO_RECOVER')
             
        if ret != EXIT_SUCCESS:
            return ret #return immediately
        else:
            print 'doEloRecover() - Task Completed! - Please check log file at '  + MUS_LOG_FILE_DIR
            TLOG.logger.info('doEloRecover() -----------------END-----------------\n')
            return ret
        
def doAipRecover(fw_path,fw_name):
        
        TLOG.logger.info('doAipRecover() -----------------START---------------')
        
        fw_path = fw_path.strip('\'')
        fw_name = fw_name.strip('\'')
        aipRecoverCmd = 'aipfwupdate'  +  ' -F  ' + str(fw_name) + ' -f'
        
        print aipRecoverCmd
        TLOG.logger.info('doAipRecover():' +  aipRecoverCmd)
        ret = runSubprocess(aipRecoverCmd,'AIP_RECOVER',str(fw_path))
        
        if (ret == ERROR_SEGFAULT or ret == ERROR_CODE_AIP_SEGFAULT or 
            ret == -11 or ret == -6) :                                  #TODO: ret == -11 or ret == -6  These error always occurs in recovery, need to investigate
            
            count = 0
            #Try to do internal recovery up to number of retries
            while(ret != 0 and count != 3):    
                txt = 'doAipRecover-----------------RETRY#'+ str(count+1) + '-----------------\n'
                TLOG.logger.info(txt)
                ret = runSubprocess(aipRecoverCmd,'VideoFwFlashTask_6149_DWU',str(fw_path))
                count += 1
                
            if ret != 0:
                    print "===================================================WARNING!======================================================"
                    
                    print "\nYou may have a attached a combination of 6149 and 4820 type monitors while performing 6149 Monitor Video Firmware.\n"
                    print "Please unplugged the attached 4820 Monitors to proceed firmware update recovery of 6149 Monitors.\n"
                    
                    TLOG.logger.info('ERROR; SEGFAULT During Recovery of Video Firmware Update, please unplugged/plugged your monitor.\n')
                    TLOG.logger.info('You may have a attached a combination of 6149 and 4820 type monitors while performing 6149 Monitor Video Firmware.\n')
                    TLOG.logger.info('Please unplugged the attached 4820 Monitors to proceed firmware update recovery of 6149 Monitors.\n')
                    
                    msg = '\"Press any key to continue...\"'
                    runShell('read -p  ' + msg + ' -n1 -s')
           
                    
        if ret != EXIT_SUCCESS:
            return ret #return immediately
        else:
            print 'doAipRecover() - Task Completed! - Please check log file at '  + MUS_LOG_FILE_DIR
            TLOG.logger.info('doAipRecover() -----------------END-----------------\n')
            return ret
        

        
def doOcularRecover(fw_path,conf):
        
        TLOG.logger.info('doOcularRecover() -----------------START---------------')
        
        fw_file_path = fw_path.rstrip('\'')
        cfg_path = conf.rstrip('\'')
     
        
        mon_list = find614Xin211Dmode()
        #flash firmware
        if (len(mon_list) != 0 ):
                for mon in mon_list:
                    
                    ocular_Recover_cmd = MXT_APP + ' -d ' + str(mon)  + ' --flash ' + fw_file_path + ' --verbose=2'            
                    print "[doOcularRecover] - This will take some time,Please wait ..."
                    
                    ret = runSubprocess(ocular_Recover_cmd,'OCULAR_RECOVER')
                    if ret != 0:
                        return ret
                        
        else:
                    ocular_Recover_cmd = MXT_APP + ' --flash ' + fw_file_path
                    ret = runSubprocess(ocular_Recover_cmd,'OCULAR_RECOVER')
        
        mon_list = find614X()
        #Load config
        if (len(mon_list) != 0 ):
                for mon in mon_list:
                    print "[doOcularRecover] - Loading Configuration now..."
                    mxt_app_load_cmd = MXT_APP +  ' -d'+ str(mon) + ' --load ' + cfg_path                                            
                    #mxt_app_load_cmd = MXT_APP + ' --load ' + cfg_path 
                    ret = runSubprocess(mxt_app_load_cmd,'MXT-APP')
               
                    mxt_app_bkup_cmd = MXT_APP + ' -W -T6 -r1 55 '
                    ret = runSubprocess(mxt_app_bkup_cmd,'MXT-APP')
                    if ret != 0:
                        return ret
                        
        
        if ret != EXIT_SUCCESS:
            return ret #return immediately
        else:
            print 'doOcularRecover() - Task Completed! - Please check log file at '  + MUS_LOG_FILE_DIR
            TLOG.logger.info('doOcularRecover() -----------------END-----------------\n')
            return ret

def updateBootCount():
    
        current_count=0
        if doesFileExist(BOOT_COUNTER) != True:
            f = open(BOOT_COUNTER,'w+')
            f.write('1')
            f.close()
        else:
            f = open(BOOT_COUNTER,'r+')
            lines = f.readlines()
            for line in lines:
                if not line:
                    break
                if line.isdigit() == True:
                    current_count = int(line.strip('\n'))
            
            #updating the counter
            f.seek(0)           # beginning of the file
            f.truncate()        # empty the file
            
            current_count += 1  # increment by 1
            
            f.write( str(current_count) )
            f.close()

def getBootCount():
        current_count=0
        if doesFileExist(BOOT_COUNTER) == True:
            f = open(BOOT_COUNTER,'r+')
            lines = f.readlines()
            for line in lines:
                if not line:
                    break
                if line.isdigit() == True:
                    current_count = int(line.strip('\n'))
                    
        return current_count